/********************************************************************************
 *
 * A very simple Garbage Collector (GC)-like implementation.
 *
 * Keeps track of the allocated memory blocks and allows to free
 * them with a single call (mostly upon exit from the program).
 *
 * This file is part of psping(1).
 *
 * Developed within the SA3-WI15 activity of the GEANT2 project.
 * http://wiki.perfsonar.net/jra1-wiki/index.php/SA3-WI15-Pages
 *
 * Copyright (c) 2007
 * Luchesar Iliev <iliev@acad.bg>
 *
 * psping is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * psping is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psping; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ********************************************************************************/


#include <stdio.h>
#include <stdlib.h>

#include "garbage-collector.h"


/********************************************************************************
 * SYNOPSYS:
 *              Initialize the Garbage Collector register.
 *
 * DESCRIPTION:
 *              Allocates memory where the Garbage Collector will keep track of
 *              the registered memory blocks. The memory is initialized with
 *              NULL pointers via calloc().
 *
 * PARAMETERS:
 *              None.
 *
 * RETURN VALUE:
 *              0
 *              if the GC was initialized successfully.
 *
 *              1
 *              if the allocation failed.
 ********************************************************************************/
int gc_init()
{
    mem_block_list = calloc(GC_MEM_BLOCK_LENGTH, sizeof(void*));
    if (mem_block_list == NULL)
    {
        fprintf(stderr, BOLD "Error:" FLAT " Failed to allocate memory for the Garbage Collector register.\n\n");
        return (1);
    }
    return (0);
}


/********************************************************************************
 * SYNOPSYS:
 *              Register a memory block.
 *
 * DESCRIPTION:
 *              Registers allocated memory block in GC's memory via its pointer.
 *              The memory block will be free()-ed when gc_mem_free() is called.
 *              The block can be unregistered (e.g. when free()-ed by the program
 *              itself) by calling gc_unreg_mem().
 *
 * PARAMETERS:
 *              void* ptr
 *              Pointer to the allocated memory block that needs to be registered.
 *
 * RETURN VALUE:
 *              0
 *              if the block/pointer was successfully registered
 *              
 *              1
 *              if the registration failed due to lack of free elements
 ********************************************************************************/
int gc_reg_mem(void* ptr)
{
    int i;  // 'for' counter
    
    // Go through the list until a free element is found; write the pointer there.
    for (i = 0; i < GC_MEM_BLOCK_LENGTH; i++)
    {
        if (mem_block_list[i] == NULL)
        {
            mem_block_list[i] = ptr;
            return (0);
        } // if
    } // for
    return (1); // no free element found to write the pointer to
}


/********************************************************************************
 * SYNOPSYS:
 *              Unregister a memory block.
 *
 * DESCRIPTION:
 *              Unregisters previously registered memory block in GC's memory.
 *              The memory block must have been previously registered via
 *              gc_reg_mem() with its pointer. NULL pointer will be written in
 *              its place, thus freeing the element for further use.
 *
 * PARAMETERS:
 *              void* ptr
 *              Pointer to memory block that needs to be unregistered.
 *
 * RETURN VALUE:
 *              0
 *              if the block/pointer was successfully unregistered
 *
 *              1
 *              if the process failed because a matching pointer was not found
 ********************************************************************************/
int gc_unreg_mem(void* ptr)
{
    int i;  // 'for' counter
    
    // Go through the list until a matching element is found; write NULL there.
    for (i = 0; i < GC_MEM_BLOCK_LENGTH; i++)
    {
        if (mem_block_list[i] == ptr)
        {
            mem_block_list[i] = NULL;
            return (0);
        } // if
    } // for
    return (1); // no matching element found
}


/********************************************************************************
 * SYNOPSYS:
 *              free() the registered memory blocks.
 *
 * DESCRIPTION:
 *              free()-s all previously registered memory blocks in GC's memory.
 *              The memory blocks must have been previously registered via
 *              gc_reg_mem() with their pointers. free() will be called for each
 *              of the pointers, and NULL will be written in its place, thus
 *              freeing all the elements for further use.
 *
 * PARAMETERS:
 *              None.
 *
 * RETURN VALUE:
 *              None
 ********************************************************************************/
void gc_free_mem()
{
    int i;  // 'for' counter
    
    // Go through the list, free()-ing all found pointers
    for (i = 0; i < GC_MEM_BLOCK_LENGTH; i++)
    {
        if (mem_block_list[i] != NULL)
        {
            free((void*) mem_block_list[i]);
            mem_block_list[i] = NULL;
        } // if
    } // for
}


/********************************************************************************
 * SYNOPSYS:
 *              Deinitialize the Garbage Collector.
 *
 * DESCRIPTION:
 *              free()-s the memory where the Garbage Collector kept track of
 *              the registered memory blocks, effectively deinitializing it.
 *              WARNING: make sure you call gc_free_mem() before calling this!
 *
 * PARAMETERS:
 *              None.
 *
 * RETURN VALUE:
 *              None
 ********************************************************************************/
void gc_deinit()
{
    free(mem_block_list);
}
