/********************************************************************************
 *
 * psping(1) : sends EchoRequests to various perfSONAR services
 * to measure availability and response times.
 *
 * Version 0.2 RC1 :: June 2007
 *
 * Developed within the SA3-WI15 activity of the GEANT2 project.
 * http://wiki.perfsonar.net/jra1-wiki/index.php/SA3-WI15-Pages
 *
 * Copyright (c) 2007
 * Luchesar Iliev <iliev@acad.bg>
 * Nickolay Landzhev <landzhev@acad.bg>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ********************************************************************************/


#include <ctype.h>
#include <curl/curl.h>
#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>

#include "psping.h"


/********************************************************************************
 * DESCRIPTION:
 *              Introduce psping and show current version.
 *
 * PARAMETERS:
 *              None.
 *
 * RETURN VALUE:
 *              None.
 ********************************************************************************/
void print_version()
{
    printf("\n");
    printf(BOLD "psping:" FLAT "\tsends EchoRequests to various perfSONAR services\n");
    printf("\tto measure availability and response times.\n");
    printf("\n");
    printf(
        "Version\t0.2 RC1 :: June 2007\n"
    );
    printf("\n");
}


/********************************************************************************
 * DESCRIPTION:
 *              Print help.
 *
 * PARAMETERS:
 *              None.
 *
 * RETURN VALUE:
 *              None.
 ********************************************************************************/
void print_help()
{
    printf("Usage:\tpsping [" BOLD "-h" FLAT "] [" BOLD "-c" FLAT " count] [" BOLD
            "-i" FLAT " interval] [" BOLD "-t" FLAT " timeout] [" BOLD "-f" FLAT "[file]] [" BOLD
            "-v" FLAT "] target\n");
    printf("\n");
    printf(BOLD "\t-h" FLAT "\tPrint this help information.\n");
    printf(BOLD "\t-c" FLAT "\tSet the number of EchoRequests to send (1-%i, defaults to %i%s).\n",
        INT_MAX, DEFAULT_REQUEST_COUNT, (DEFAULT_REQUEST_COUNT == -1) ? "\b\bunlimited" : "");
    printf(BOLD "\t-i" FLAT "\tSet the interval between two EchoRequests in seconds (0-%i, defaults to %i).\n",
        MAX_REQUEST_INTERVAL, DEFAULT_REQUEST_INTERVAL);
    printf(BOLD "\t-t" FLAT "\tSet the time to wait for an EchoResponse in seconds (1-%i, defaults to %i).\n",
        MAX_REQUEST_TIMEOUT, DEFAULT_REQUEST_TIMEOUT);
    printf(BOLD "\t-f" FLAT "\tRead the EchoRequest message from " BOLD "file" FLAT " (defaults to \"message.xml\").\n");
    printf(BOLD "\t\tWARNING:" FLAT " There must be NO whitespace between " BOLD "-f" FLAT " and " BOLD "file" FLAT ".\n");
    printf(BOLD "\t-v" FLAT "\tBe verbose (use two or three times for greater detail).\n");
    printf("\n");
    printf(BOLD "\ttarget" FLAT "\tThe full URL of the service to send the EchoRequest to.\n");
    printf("\n");
}


/********************************************************************************
 * DESCRIPTION:
 *              Complain about not being able to allocate a memory block.
 *
 * PARAMETERS:
 *              const char* purpose
 *              Pointer to string containing explanation what the memory block
 *              that failed to register would have been used for (e.g. buffer).
 *
 * RETURN VALUE:
 *              None.
 ********************************************************************************/
void mem_complain(const char* purpose)
{
    fprintf(stderr, BOLD "Error:" FLAT " Failed to allocate memory for %s.\n", purpose);
}


/********************************************************************************
 * DESCRIPTION:
 *              Generic complaint about error.
 *
 * PARAMETERS:
 *              const char* explanation
 *              Pointer to string containing explanation of the error.
 *
 * RETURN VALUE:
 *              None.
 ********************************************************************************/
void err_complain(const char* explanation)
{
    fprintf(stderr, BOLD "Error:" FLAT " %s\n", explanation);
}


/********************************************************************************
 * SYNOPSYS:
 *              Initialize psping variables.
 *
 * DESCRIPTION:
 *              Initialize psping variables with the default values (most of them
 *              from psping.h). The char* variables get memory blocks from calloc().
 *              calloc() allows to easily set the required bytes aside considering
 *              the size of the char variable. It also initializes the bytes to
 *              '\000', which helps in certain cases where strings are handled.
 *
 * PARAMETERS:
 *              psping_vars_t* vars
 *              Pointer to the structure comprised of all relevant psping variables.
 *              The structure is defined in psping.h.
 *
 * RETURN VALUE:
 *              0 if variables were sucessfully initialized
 *              1 if any of the memory allocations failed
 ********************************************************************************/
int init_vars(psping_vars_t* vars)
{
    // Most defaults are defined in psping.h.
    vars->count = DEFAULT_REQUEST_COUNT;
    vars->interval = DEFAULT_REQUEST_INTERVAL;
    vars->timeout = DEFAULT_REQUEST_TIMEOUT;
    vars->verbose_flag = 0;

    // For pointers we use calloc(): it fills the referenced memory with \000's
    // and the right number of bytes are reserved for the string via sizeof(char).
    // We allocate one additional element for the null-character at the end.
    vars->service_address = (char*) calloc(MAX_SERVICE_ADDRESS_LENGTH + 1, sizeof(char));
    if (vars->service_address == NULL)
    {
        mem_complain("service address");
        return (1);
    }
    gc_reg_mem(vars->service_address);  // register with the Garbage Collector
    strcpy(vars->service_address, DEFAULT_SERVICE_ADDRESS);

    vars->request_message = (char*) calloc(MAX_REQUEST_LENGTH + 1, sizeof(char));
    if (vars->request_message == NULL)
    {
        mem_complain("request message");
        return (1);
    }
    gc_reg_mem(vars->request_message);  // register with the Garbage Collector
    strcpy(vars->request_message, DEFAULT_REQUEST_MESSAGE);

    vars->request_filename = (char*) calloc(MAX_FILENAME_LENGTH + 1, sizeof(char));
    if (vars->request_filename == NULL)
    {
        mem_complain("request_filename");
        return (1);
    }
    gc_reg_mem(vars->request_filename); // register with the Garbage Collector
    strcpy(vars->request_filename, DEFAULT_REQUEST_FILENAME);

    vars->response_message = (char*) calloc(MAX_RESPONSE_LENGTH + 1, sizeof(char));
    if (vars->response_message == NULL)
    {
        mem_complain("response message");
        return (1);
    }
    gc_reg_mem(vars->response_message); // register with the Garbage Collector

    return (0);
}


/********************************************************************************
 * SYNOPSYS:
 *              Parse the command line.
 *
 * DESCRIPTION:
 *              Parses the command-line options and initializes the relevant
 *              variables. This includes reading the file supplied by the -f
 *              option and validation of the target URL.
 *
 * PARAMETERS:
 *              int argc
 *              A count of the arguments supplied to the program.
 *
 *              char** argv
 *              An array of pointers to the strings which are those arguments.
 *
 *              const char* valid_options
 *              The const char *options argument to getopt().
 *
 *              psping_vars_t* vars
 *              Pointer to the structure containing the psping variables to be
 *              initialized in accordance with the command-line options. The
 *              structure is defined in psping.h.
 *
 * RETURN VALUE:
 *              0
 *              if the parsing was successful and the variables have been
 *              initialized.
 *
 *              1
 *              if there was a problem, like invalid option.
 *
 *              -1
 *              if help was requested (it's not a failure, yet the program must
 *              still exit).
 ********************************************************************************/
int parse_cmd_line(int argc, char** argv, const char* valid_options, psping_vars_t* vars)
{
    int option; // used to evaluate each of the command line options

    while ((option = getopt(argc, argv, valid_options)) != -1)
    {
        switch (option)
        {
            // Help requested
            case 'h':
                print_help();
                return (-1);
                break;

            // The number of EchoRequests to send specified
            case 'c':
                if (isdigit((int) *optarg))
                {
                    vars->count = atoi(optarg);
                }
                else
                {
                    err_complain("You must supply a numeric argument as number of EchoRequests to send.");
                    print_help();
                    return (1);
                }

                if (vars->count < 1)
                {
                    fprintf(stderr, BOLD "Error:" FLAT " The number of EchoRequests to send (%i) is invalid.\n\n", vars->count);
                    print_help();
                    return (1);
                }
                break;

            // The interval between two EchoRequests specified
            case 'i':
                if (isdigit((int) *optarg))
                {
                    vars->interval = atoi(optarg);
                }
                else
                {
                    err_complain("You must supply a numeric argument as interval between the EchoRequests.");
                    print_help();
                    return (1);
                }

                if (vars->interval < 0 || vars->interval > MAX_REQUEST_INTERVAL)
                {
                    fprintf(stderr, BOLD "Error:" FLAT " The interval between the EchoRequests (%i) is out of range.\n\n", vars->interval);
                    print_help();
                    return (1);
                }
                break;

            // The time to wait for an EchoResponse specified
            case 't':
                if (isdigit((int) *optarg))
                {
                    vars->timeout = atoi(optarg);
                }
                else
                {
                    err_complain("You must supply a numeric argument as time to wait for an EchoResponse.");
                    print_help();
                    return (1);
                }

                if (vars->timeout < 1 || vars->timeout > MAX_REQUEST_TIMEOUT)
                {
                    fprintf(stderr, BOLD "Error:" FLAT " The time to wait for an EchoResponse (%i) is out of range.\n\n", vars->timeout);
                    print_help();
                    return (1);
                }
                break;

            // File to read the EchoRequest message from instead of using the hardcoded message
            // If we have a filename provided, use it, otherwise stick to default "message.xml"
            // Beware, the filename must follow "-f" *without* space, e.g. "-fmessage.xml"!
            case 'f':
                if (optarg != NULL)
                {
                    strcpy(vars->request_filename, optarg);
                }
                // If everything is OK, request_message will have the file contents in it
                if (read_message_file(vars->request_filename, vars->request_message) != 0)
                {
                    err_complain("File read failed.");
                    return (1);
                }
                break;

            // Verbose level selected
            case 'v':
                vars->verbose_flag++;
                break;

            // All unknown options lead here
            case '?':
                if (isprint(optopt))
                {
                    fprintf(stderr, "Unknown option `-%c'.\n\n", optopt);
                    print_help();
                }
                else
                {
                    fprintf(stderr, "Unknown option character `\\x%x'.\n\n", optopt);
                    print_help();
                }
                return (1);
                break;

            // Just in case
            default:
                abort ();
                break;
        }   // switch
    }   // while

    // Do we have target URL provided on the command line?
    if (argv[optind] == NULL)
    {
        err_complain("You must specify target URL.");
        print_help();
        return (1);
    }

    // Validate the URL
    if (validate_url(argv[optind]) != 0)
    {
        err_complain("URL validation failed.");
        return (1);
    }

    // Isn't the URL provided too long to fit in our array?
    if (strlen(argv[optind]) > MAX_SERVICE_ADDRESS_LENGTH)
    {
        err_complain("Target URL too long.");
        return (1);
    }

    // OK, read the URL
    strcpy(vars->service_address, argv[optind]);

    return (0);
}


/********************************************************************************
 * SYNOPSYS:
 *              Read the file containing the custom EchoRequest message.
 *
 * DESCRIPTION:
 *              Opens the file set by the -f option, and reads its contents into
 *              the request_message memory block (which gets cleared from the
 *              default message beforehands). Checks for possible buffer
 *              overflows and processes any file errors encountered.
 *
 * PARAMETERS:
 *              const char* request_filename
 *              The name of the file containing the request.
 *
 *              char* request_message
 *              The memory to write the read message to.
 *
 * RETURN VALUE:
 *              0
 *              on success
 *
 *              1
 *              on failure
 ********************************************************************************/
int read_message_file(const char* request_filename, char* request_message)
{
    FILE* file_handle;      // pointer to the FILE object for the request_filename
    long int file_size;     // the size of the file in bytes

    // Open the file for reading
    if ((file_handle = fopen(request_filename, "r")) == NULL)
    {
        fprintf (stderr, "filename: %s\n", request_filename);
        err_complain("Failed to open the file. Does it exist? Is it readable?");
        return (1);
    }

    // Determine filesize
    fseek(file_handle, 0, SEEK_END);
    file_size = ftell(file_handle);
    rewind(file_handle);

    // Check if the filesize is not too large to fit in request_message
    if (file_size > (MAX_REQUEST_LENGTH * sizeof(char)))
    {
        fprintf (stderr, "filename: %s\n", request_filename);
        err_complain("File is too large to fit in memory.");
        fclose(file_handle);
        return (1);
    }

    // Clean up the default request_message (one more (char) for the null-character)
    memset(request_message, 0, file_size + sizeof(char));

    // Read the file contents into request_message and check for errors
    if ((fread(request_message, 1, file_size, file_handle) != file_size) && (ferror(file_handle) != 0))
    {
        fprintf (stderr, "filename: %s\n", request_filename);
        process_file_error(errno);  // errno is defined in errno.h
        fclose(file_handle);
        return (1);
    }

    // Everything was OK
    fclose(file_handle);
    return (0);
}


/********************************************************************************
 * SYNOPSYS:
 *              Process and complain about a file error
 *
 * DESCRIPTION:
 *              Analyze errno (errno.h) code and print out a relevant message.
 *
 * PARAMETERS:
 *              int error_code
 *              The error code to be interpreted.
 *
 * RETURN VALUE:
 *              None.
 ********************************************************************************/
void process_file_error(int error_code)
{
    switch (error_code)
    {
        case EACCES:
            err_complain("Another process has the file locked.");
            break;

        case EBADF:
            err_complain("Stream is not a valid stream opened for reading.");
            break;

        case EINTR:
            err_complain("A signal interrupted the call.");
            break;

        case EIO:
            err_complain("An input error occurred.");
            break;

        case EISDIR:
            err_complain("The open object is a directory, not a file.");
            break;

        case ENOMEM:
            err_complain("Memory could not be allocated for internal buffers.");
            break;

        case ENXIO:
            err_complain("A device error occurred.");
            break;

        case EOVERFLOW:
            err_complain("The file is a regular file and an attempt was made to read at or beyond the offset maximum associated with the corresponding stream.");
            break;

        case EWOULDBLOCK:
            err_complain("The underlying file descriptor is a non-blocking socket and no data is ready to be read.");
            break;

        default:
            err_complain("Unknown error.");
            break;
    }
}


/********************************************************************************
 * SYNOPSYS:
 *              Write the data received from CURL into a buffer.
 *
 * DESCRIPTION:
 *              This function gets called by libcurl as soon as there is data
 *              received that needs to be saved. 
 *
 * PARAMETERS:
 *              const void* recv_data
 *              Points to the received data that CURL needs to be saved.
 *
 *              size_t size
 *              The size of each element of the data pointed by recv_data.
 *
 *              size_t nmemb
 *              The number of elements in the received data.
 *
 *              void* recv_buffer
 *              Pointer to the buffer where the data must be saved. It is set in
 *              request_response() to point to (psping_vars_t).response_message via
 *              curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, response_message);
 *
 * RETURN VALUE:
 *              size_t
 *              The number of bytes actually taken care of.
 ********************************************************************************/
size_t write_curl_data(const void* recv_data, size_t size, size_t nmemb, void* recv_buffer)
{
    size_t data_size;   // the total "raw" size of the received data

    data_size = nmemb * size;
    memcpy(recv_buffer, recv_data, data_size);
    return data_size;
}


/********************************************************************************
 * SYNOPSYS:
 *              Send EchoRequest, receive EchoResponse, and calculate the delay.
 *
 * DESCRIPTION:
 *              TBA
 *
 * PARAMETERS:
 *              const char* service_address
 *              The address of the tested service.
 *
 *              const char* request_message
 *              The message sent to the service.
 *
 *              int timeout
 *              The time to wait for an EchoResponse in seconds.
 *
 *              int verbose_flag
 *              The current level of verbosity. If > 1, cURL's communications
 *              are printed out. If >= 3, the message sent is printed as well.
 *
 *              char* response_message
 *              Returns the message received from the host/service.
 *
 *              double* elapsed_time
 *              Returns the delay between the EchoRequest and the EchoResponse.
 *
 * RETURN VALUE:
 *              0
 *              on success
 *
 *              1
 *              on failure
 ********************************************************************************/
int request_response(const char* service_address,
                     const char* request_message,
                     int timeout,
                     int verbose_flag,
                     char* response_message,
                     double* elapsed_time)
{
    CURL* curl_handle;                  // handle for the connection given by CURL
    CURLcode curl_result;               // result returned by the CURL request
    struct curl_slist* http_headers;    // the HTTP headers CURL will send
    struct timeval current_time;        // the current time at the moment
    double request_timestamp;           // the moment of time we send the EchoRequest
    double response_timestamp;          // the moment of time we receive the EchoResponse

    // Initialize CURL
    curl_handle = curl_easy_init();
    if (curl_handle == NULL)
    {
        err_complain("Could not initialize CURL.");
        return (1);
    }

    // Enable verbose mode if requested (1+ would mean enabled)
    curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, verbose_flag);

    // Set the target
    curl_easy_setopt(curl_handle, CURLOPT_URL, service_address);

    // Set the function that will handle the data returned from CURL
    curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, &write_curl_data);

    // Set where to write the data returned from CURL
    curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, response_message);

    // Set the timeout
    curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, timeout);

    // Set the message to be sent
    curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, request_message);

    // Set the HTTP headers
    http_headers = NULL;
    http_headers = curl_slist_append(http_headers, "Content-Type: text/xml");
    http_headers = curl_slist_append(http_headers, DEFAULT_SOAPACTION);
    curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, http_headers);

    // Print out the sent message if required by the verbosity level
    if (verbose_flag >= 3)
    {
        printf("\n" BOLD "Sent data dump follows:" FLAT "\n%s\n", request_message);
    }
    else if (verbose_flag > 0)
    {
        printf("\n");
    }

    // What's the time?
    gettimeofday(&current_time, NULL);
    request_timestamp = current_time.tv_sec + (current_time.tv_usec/1000000.0);

    // Execute the request and receive a response
    curl_result = curl_easy_perform(curl_handle);
    if (curl_result != 0)
    {
        err_complain("CURL returned non-successful status.");
        return (1);
    }

    // What's the time?
    gettimeofday(&current_time, NULL);
    response_timestamp = current_time.tv_sec + (current_time.tv_usec/1000000.0);

    // Calculate the delay
    *elapsed_time = response_timestamp - request_timestamp;

    // Free the curl_slist made with curl_slist_append()
    curl_slist_free_all(http_headers);

    // Close all connections this handle has used and possibly has kept open until now
    curl_easy_cleanup(curl_handle);

    return (0);
}


/********************************************************************************
 * SYNOPSYS:
 *              Extract substring from a string.
 *
 * DESCRIPTION:
 *              Extracts substring identified by surrounding substrings.
 *
 * PARAMETERS:
 *              const char* string
 *              The string to be parsed for extraction.
 *
 *              const char* pre_substr
 *              The substring preceeding the desired substring to be extracted.
 *
 *              const char* post_substr
 *              The substring following the desired substring to be extracted.
 *
 * RETURN VALUE:
 *              char*
 *              Pointer to the extracted substring.
 ********************************************************************************/
char* str_extract(const char* string, const char* pre_substr, const char* post_substr)
{
    char* extraction_start; // pointer to the first char to be extracted
    int extraction_length;  // the length of the substring to be extracted
    char* buffer;           // the buffer to write the extracted substring to

    // The position where we find the pre_substr, incremented with the length of
    // pre_substr itself, gives us exactly the first char of the substring to extract.
    extraction_start = strstr(string, pre_substr) + strlen(pre_substr);

    // The position where post_substr begins and thus where the extraction part ends
    // minus the extraction_start position gives exactly the length of the extraction part.
    extraction_length = strstr(string, post_substr) - extraction_start;

    // calloc() initializes memory to '\000', so +1 element ensures that we have
    // a properly terminated string there.
    buffer = (char*) calloc(extraction_length + 1, sizeof(char));
    if (buffer == NULL) {
        mem_complain("parsing");
        return(NULL);
    }
    // Register the block with the Garbage Collector
    gc_reg_mem(buffer);

    // Remember, we have an additional '\000' element at the end.
    strncpy(buffer, extraction_start, extraction_length);

    return (buffer);
}


/********************************************************************************
 * SYNOPSYS:
 *              Parse the EchoResponse message.
 *
 * DESCRIPTION:
 *              Analyze the EchoResponse message to find certain patterns which
 *              indicate the service's condition. Print out an appropriate
 *              message, including the elapsed time if everything is OK.
 *
 * PARAMETERS:
 *              const char* response_message
 *              The EchoResponse message to be analyzed.
 *
 *              double elapsed_time
 *              The EchoRequest delay to be printed if success.echo is returned.
 *
 *              int verbose_flag
 *              If set to >= 2 will print the EchoResponse message.
 *
 * RETURN VALUE:
 *              0
 *              if the analysis showed positive result (success.echo)
 *
 *              1
 *              if the analysis showed problem in the response (faultcode, etc.)
 ********************************************************************************/
int parse_response(const char* response_message, double elapsed_time, int verbose_flag)
{
    if (response_message == NULL)
    {
        err_complain("No connection with this service.");
    }
    else if (strstr(response_message, "404 Not Found") != NULL)
    {
        err_complain("The requested URL was not found on this server.");
        return (1);
    }
    else if (strstr(response_message, "</faultcode>") != NULL)
    {
        fprintf(stderr, BOLD "Error:" FLAT " %s\n", str_extract(response_message, "<faultstring>", "</faultstring>"));

        if (verbose_flag >= 2)
        {
            printf("\n" BOLD "Received data dump follows:" FLAT "\n%s\n", response_message);
        }

        return (1);
    }
    else if (strstr(response_message, "Action [EchoRequest]") != NULL)
    {
        err_complain("Action [EchoRequest] is not supported by the service or cannot be served: cannot instantiate Message Handler.\n");

        if (verbose_flag >= 2)
        {
            printf("\n" BOLD "Received data dump follows:" FLAT "\n%s\n", response_message);
        }

        return (1);
    }
    else if (strstr(response_message, "success.echo") != NULL)
    {
        printf("Elapsed time: " BOLD "%.6lf" FLAT " seconds\n", elapsed_time);

        if (verbose_flag == 1)
        {
            printf("\n- - - 8< - - - - - - - - - - - - - - -\n");
        }

        if (verbose_flag >= 2)
        {
            printf("\n" BOLD "Received data dump follows:" FLAT "\n%s\n", response_message);
            printf("\n- - - 8< - - - - - - - - - - - - - - -\n");
        }
        return (0);
    }
    else
    {
        err_complain("Not a perfSONAR service or received data not understood.");

        if (verbose_flag >= 2)
        {
            printf("\n" BOLD "Received data dump follows:" FLAT "\n%s\n", response_message);
        }
        return (1);
    }
    return (1);
}


/********************************************************************************
 * A simple function to print out the current values of the psping variables
 ********************************************************************************/
#ifdef DEBUG
void print_vars(const psping_vars_t* vars)
{
    printf("count:\t\t\t%i\n\n", vars->count);
    printf("interval:\t\t%i\n\n", vars->interval);
    printf("timeout:\t\t%i\n\n", vars->timeout);
    printf("verbose_flag:\t\t%i\n\n", vars->verbose_flag);
    printf("service address:\t%s\n\n", vars->service_address);
    printf("request_filename:\t%s\n\n", vars->request_filename);
    printf("request_message:\n%s\n\n", vars->request_message);
}
#endif


/********************************************************************************
 ********************************************************************************
 **
 ** MAIN
 **
 ********************************************************************************
 ********************************************************************************/

int main(int argc, char** argv)
{
    psping_vars_t vars;  //structure containing all psping variables (see psping.h)


    // Do we have command line parameters? If we don't, it's pointless to continue.
    if (argc == 1)
    {
        print_version();
        print_help();
        return (EXIT_FAILURE);
    }

    // Initialize the Garbage Collector
    if (gc_init() != 0)
    {
        err_complain("Could initialize the Garbage Collector.");
        return (EXIT_FAILURE);
    }

    // Initialize variables
    if (init_vars(&vars) != 0)
    {
        err_complain("Could initialize the variables.");
        // Clean up with the Garbage Collector
        gc_free_mem();
        gc_deinit();
        return (EXIT_FAILURE);
    }

    // Parse the command line
    switch (parse_cmd_line(argc, argv, "hc:i:t:f::v", &vars))
    {
        // Parsing successful; continue with the program
        case 0:
            break;

        // Parsing failed; fail gracefully
        case 1:
            // Clean up with the Garbage Collector
            gc_free_mem();
            gc_deinit();
            return (EXIT_FAILURE);
            break;

        // Help requested; exit normally
        case -1:
            // Clean up with the Garbage Collector
            gc_free_mem();
            gc_deinit();
            return (EXIT_SUCCESS);
            break;

        // Something very wrong happened; bail out
        default:
            // Clean up with the Garbage Collector
            gc_free_mem();
            gc_deinit();
            abort();
            break;
    }
    
    // This is some debugging code to check for proper variable initialization
#ifdef DEBUG
    print_vars(&vars);
#endif

    // Everything is set up, so let's do the job this is all about
    // Check whether the service responds and with what delay
    do
    {
        if (request_response(vars.service_address,
                             vars.request_message,
                             vars.timeout,
                             vars.verbose_flag,
                             vars.response_message,
                             &vars.elapsed_time) != 0)
        {
            err_complain("Could not execute the request.");
            // Clean up with the Garbage Collector
            gc_free_mem();
            gc_deinit();
            return (EXIT_FAILURE);
        }

        if (parse_response(vars.response_message, vars.elapsed_time, vars.verbose_flag) != 0)
        {
            // Clean up with the Garbage Collector
            gc_free_mem();
            gc_deinit();
            return (EXIT_FAILURE);
        }

        // One count is over, therefore decrement counter, unless vars.count is set to infinity
        if (vars.count != -1) vars.count--;

        if ((vars.count != 0) && (vars.interval > 0))   // do we need to sleep() between the requests?
        {
            if (vars.verbose_flag > 0)
            {
                printf("\nSleeping for %i second(s)...\n", vars.interval);
            }
            sleep(vars.interval);
        }

    } while (vars.count != 0);  // if vars.count is -1, we'll continue endlessly (until SIGINT, that is)

    // Clean up with the Garbage Collector
    gc_free_mem();
    gc_deinit();

    // Phewww, it's over, so we can have a well-deserved rest ;)
    return (EXIT_SUCCESS);

}   //main
