/********************************************************************************
 *
 * Copyright (c) 2007
 * Luchesar Iliev <iliev@acad.bg>
 * Nickolay Landzhev <landzhev@acad.bg>
 *
 * This file is part of psping(1).
 *
 * psping is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * psping is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psping; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ********************************************************************************/


// HEADER GUARD
#ifndef PSPING__PSPING_H
#define PSPING__PSPING_H


/********************************************************************************
 ********************************************************************************
 **
 ** Things you can change
 **
 ********************************************************************************
 ********************************************************************************/

// Default number of EchoRequests to send (-1 means unlimited)
#define DEFAULT_REQUEST_COUNT       -1

// Default interval between two EchoRequests in seconds
#define DEFAULT_REQUEST_INTERVAL    1

// Maximum interval between two EchoRequests in seconds
#define MAX_REQUEST_INTERVAL        3600

// Default time to wait for an EchoResponse in seconds
#define DEFAULT_REQUEST_TIMEOUT     3

// Maximum time to wait for an EchoResponse in seconds
#define MAX_REQUEST_TIMEOUT         60

// Maximum length of the URL parsed from the command line in symbols
#define MAX_SERVICE_ADDRESS_LENGTH  200

// Default service address
// WARNING: This must be NO LONGER than MAX_SERVICE_ADDRESS_LENGTH!
#define DEFAULT_SERVICE_ADDRESS     "http://127.0.0.1:8080/axis/services/MeasurementArchiveService"

// Maximum length of the name of the file containing the EchoRequest message
#define MAX_FILENAME_LENGTH         200

// Default name of the file containing the EchoRequest message
// WARNING: This must be NO LONGER than MAX_FILENAME_LENGTH!
#define DEFAULT_REQUEST_FILENAME    "message.xml"

// Maximum length of the EchoRequest message in symbols
#define MAX_REQUEST_LENGTH          10000

// Default SOAPAction for the EchoRequest
#define DEFAULT_SOAPACTION          "SOAPAction: \"\""

// Default message to send to the service
// Don't forget the "\" at the end of the wrapped lines
// WARNING: This must be NO LONGER than MAX_REQUEST_LENGTH
#define DEFAULT_REQUEST_MESSAGE     \
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
<s:Envelope\n\
\txmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"\n\
\txmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\"\n\
\txmlns:xsd=\"http://www.w3.org/1999/XMLSchema\">\n\
\t<s:Body>\n\
\t\t<nmwg:message type=\"EchoRequest\" id=\"id1\" xmlns:nmwg=\"http://ggf.org/ns/nmwg/base/2.0/\">\n\
\t\t\t<nmwg:metadata id=\"meta\">\n\
\t\t\t\t<nmwg:eventType>\n\
\t\t\t\t\thttp://schemas.perfsonar.net/tools/admin/echo/2.0\n\
\t\t\t\t</nmwg:eventType>\n\
\t\t\t</nmwg:metadata>\n\
\t\t\t<nmwg:data id=\"data\" metadataIdRef=\"meta\"/>\n\
\t\t</nmwg:message>\n\
\t</s:Body>\n\
</s:Envelope>\n"

// Maximum length of the EchoResponse message in symbols
// WARNING: This must be large enough to accommodate the whole message
#define MAX_RESPONSE_LENGTH          10000


/********************************************************************************
 ********************************************************************************
 **
 ** Things you should NOT change
 **
 ********************************************************************************
 ********************************************************************************/

/*
    DEFINES
*/

// Bold and normal attributes for terminal output
#define BOLD                    "\033[1m"
#define FLAT                    "\033[m"

/*
    TYPEDEFS
*/

// Structure for passing the psping variables to functions
typedef struct
{
    int count;                  // the number of EchoRequests to send (-1 for unlimited)
    int interval;               // the interval between two EchoRequests in seconds
    int timeout;                // the time to wait for an EchoResponse in seconds
    int verbose_flag;               // if set to 1, details about the connection status are printed
                                    // if set to 2, the EchoResponse message is printed in addition
                                    // if set to 3, the EchoRequest message is printed too
    char* service_address;      // the address of the tested service
    char* request_message;      // the message sent to the service
    char* request_filename;     // the name of the file containing the EchoRequest message
    char* response_message;     // the message received from the host/service
    double elapsed_time;        // the delay between the EchoRequest and the EchoResponse
} psping_vars_t;

/*
    FUNCTIONS
*/

// Introduce psping and show current version
void print_version();

// Print help
void print_help();

// Complain about not being able to allocate a memory block
void mem_complain(const char*);

// Generic complaint about error
void err_complain(const char*);

// Initialize psping variables
int init_vars(psping_vars_t*);

// Parse the command line
int parse_cmd_line(int, char**, const char*, psping_vars_t*);

// Read the file containing the custom EchoRequest message
int read_message_file(const char*, char*);

// Process and complain about a file error
void process_file_error(int);

// Write the data received from CURL into a buffer
size_t write_curl_data(const void*, size_t, size_t, void*);

// Send EchoRequest, receive EchoResponse, and calculate the delay
int request_response(const char*, const char*, int, int, char*, double*);

// Extract substring from a string
char* str_extract(const char*, const char*, const char*);

// Parse the EchoResponse message
int parse_response(const char*, double, int);

// A simple function to print out the current values of the psping variables
#ifdef DEBUG
void print_vars(const psping_vars_t*);
#endif

/*
    EXTERNAL FUNCTIONS
*/

// Initialize the Garbage Collector register
extern int gc_init();

// Register a memory block
extern int gc_reg_mem(void*);

// Unregister a memory block
extern int gc_unreg_mem(void*);

// free() the registered memory blocks
extern void gc_free_mem();

// Deinitialize the Garbage Collector
extern void gc_deinit();

// Validate URL against a preset pattern
extern int validate_url(const char*);


// HEADER GUARD
#endif                          // PSPING__PSPING_H

