package probes::perfSONARping;

my $DEFAULTBIN = "/usr/bin/psping";

=head1 NAME

probes::perfSONARping - a psping(1) probe for SmokePing

=head1 OVERVIEW

Measures perfSONAR service roundtrip times for SmokePing.

=head1 SYNOPSYS

 *** Probes ***
 + perfSONARping

 binary = /usr/bin/psping # default value

 *** Targets ***

 probe = perfSONARping
 forks = 10

 menu = Top
 title = Top Menu
 remark = Top Menu Remark

 + PROBE_CONF

 url = /
 # none of these are mandatory
 port = 80
 timeout = 1
 interval = 1
 
 + First
 menu = First
 title = First Target
 host = host.example.com

 # PROBE_CONF can be overridden here
 ++ PROBE_CONF
 port = 8080

=head1 DESCRIPTION

Supported probe-specific variables:

=over

=item binary

The location of your psping binary.

=item forks

The number of concurrent processes to be run. See probes::basefork(3pm) 
for details.

=back

Supported target-level probe variables 
(see psping(1) for details of the options):

=over

=item url

The path part of the URL of the perfSONAR service.

=item port

The TCP port to use. The default is 80.

=item timeout

The "-t" psping(1) option. 

=item interval

The "-i" psping(1) option. 

=item extraopts

Any extra options specified here will be passed unmodified to psping(1).

=back

=head1 BUGS

TBA

=head1 AUTHOR

Luchesar V. ILIEV E<lt>iliev@acad.bgE<gt>

Based on EchoPing and EchoPingHttp probes by
Niko Tyni E<lt>ntyni@iki.fiE<gt>

=head1 SEE ALSO

psping(1), http://www.perfsonar.net/

=cut

use strict;
use base qw(probes::basefork);
use Carp;
#
# derived class will mess with this through the 'features' method below
my $featurehash = {
	interval => "-i",
	timeout => "-t",
	count => "-c",
};

sub features {
	my $self = shift;
	my $newval = shift;
	$featurehash = $newval if defined $newval;
	return $featurehash;
}

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = $class->SUPER::new(@_);

	$self->_init if $self->can('_init');

	unless (defined $self->{properties}{binary}) {
		$self->{properties}{binary} = $DEFAULTBIN;
	}
	croak "ERROR: psping 'binary' $self->{properties}{binary} does not point to an executable"
		unless -f $self->{properties}{binary} and -x $self->{properties}{binary};

	$self->test_usage;

	return $self;
}

sub make_host_init {
        my $self = shift;
        my $target = shift;
        return $target->{addr};
}

sub make_args {
	my $self = shift;
	my $target = shift;
	my @args;
	my %arghash = %{$self->features};

	for (keys %arghash) {
		my $val = $target->{vars}{$_};
		$val = $self->{properties}{$_} unless defined $val;
		push @args, ($arghash{$_}, $val) if defined $val;
	}
	push @args, $self->{properties}{extraopts} if exists $self->{properties}{extraopts};
	push @args, $target->{vars}{extraopts} if exists $target->{vars}{extraopts};

	return @args;
}

# this is separated to make it possible to test the service
# at startup, although we don't do it at the moment.
sub count_args {
	my $self = shift;
	my $count = shift;

	$count = $self->pings() unless defined $count;
	return ("-c", $count);
}

sub make_commandline {
	my $self = shift;
	my $target = shift;
	my $count = shift;

	$count |= $self->pings($target);

	my @args = $self->make_args($target);
	push @args, $self->count_args($count);

	my $host = $self->make_host($target);
	
	return ($self->{properties}{binary}, @args, $host);
}

sub pingone {
	my $self = shift;
	my $t = shift;

	my @cmd = $self->make_commandline($t);

	my $cmd = join(" ", @cmd);

	$self->do_debug("executing cmd $cmd");

	my @times;

	open(P, "$cmd 2>&1 |") or carp("fork: $!");
	
	# what should we do with error messages?
	my $echoret;
	while (<P>) {
		$echoret .= $_;
		/^Elapsed time: (\d+\.\d+) seconds/ and push @times, $1;
	}
	close P;
	carp "WARNING: $cmd was not happy: $echoret\n" if $?;
	# carp("Got @times") if $self->debug;
	return sort { $a <=> $b } @times;
}

sub make_host {
        my $self = shift;
        my $target = shift;

        my $host = $target->{addr};

        my $url = $target->{vars}{url};
        $url = $self->{properties}{url} unless defined $url;
        $url = "/" unless defined $url;

        my $port = $target->{vars}{port};
        $port = $self->{properties}{port} unless defined $port;

        $host = "http://" . $host;
        $host .= ":$port" if defined $port;
        $host .= $url if defined $url;

        return $host;
}

# warn about unsupported features
sub test_usage {
        my $self = shift;
        my $bin = $self->{properties}{binary};
        my @unsupported;

        my $arghashref = $self->features;
        my %arghash = %$arghashref;

        for my $feature (keys %arghash) {
                if (`$bin $arghash{$feature} 1 http://127.0.0.1/ 2>&1` =~ /unknown option|usage/i) {
                        push @unsupported, $feature;
                        $self->do_log("Note: your psping doesn't support the $feature feature (option $arghash{$feature}), disabling it");
                }
        }
        map { delete $arghashref->{$_} } @unsupported;

        return;
}

sub ProbeDesc($) {
        return "perfSONAR EchoRequests using psping(1)";
}

1;
