/********************************************************************************
 *
 * A simple set of validation functions.
 *
 * This file is part of psping(1).
 *
 * Developed within the SA3-WI15 activity of the GEANT2 project.
 * http://wiki.perfsonar.net/jra1-wiki/index.php/SA3-WI15-Pages
 *
 * Copyright (c) 2007
 * Luchesar Iliev <iliev@acad.bg>
 *
 * psping is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * psping is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psping; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ********************************************************************************/


#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "validation.h"


/********************************************************************************
 * SYNOPSYS:
 *              Validate URL against a preset pattern.
 *
 * DESCRIPTION:
 *              Checks the URL supplied on the command-line against the pattern
 *              preset in validation.h to ensure that it is indeed a valid URL.
 *              Uses validate_string() for this purpose.
 *
 * PARAMETERS:
 *              const char* url
 *              The URL to be tested. Must be a valid C string.
 *
 * RETURN VALUE:
 *              0
 *              on successful validation.
 *
 *              1
 *              on unsuccessful validation.
 *
 *              2
 *              if a problem with REGEX initialization was encountered.
 *
 *              3
 *              if there was a problem allocating memory for the pattern.
 ********************************************************************************/
int validate_url(const char* url)
{
    char* url_pattern;      // valid URL pattern
    int validation_result;  // exit status from string_validate()

    // Initialize the pattern
    url_pattern = (char*) malloc(strlen(URL_VALID_PATTERN));
    if (url_pattern == NULL)
    {
        fprintf(stderr, BOLD "Error:" FLAT " Failed to allocate memory for the URL pattern.\n\n");
        return (3);
    }
    strcpy(url_pattern, URL_VALID_PATTERN);

    // Validate the URL
    validation_result = validate_string(url, url_pattern);

    free(url_pattern);
    return (validation_result);
}


/********************************************************************************
 * SYNOPSYS:
 *              Validate a string against a preset pattern.
 *
 * DESCRIPTION:
 *              Checks the string supplied on the command-line against the supplied
 *              pattern using the functions from regex.h.
 *
 * PARAMETERS:
 *              const char* string
 *              A valid C string to be tested.
 *
 * RETURN VALUE:
 *              0
 *              on successful validation.
 *
 *              1
 *              on unsuccessful validation.
 *
 *              2
 *              if a problem with REGEX initialization was encountered.
 ********************************************************************************/
int validate_string(const char* string, const char* pattern)
{
    int status;     // exit status from regexec()
    regex_t re;

    // Initialize the regular expression
    if (regcomp(&re, pattern, REG_EXTENDED|REG_ICASE|REG_NOSUB) != 0)
    {
        fprintf(stderr, BOLD "Error:" FLAT " Failed to initialize REGEX.\n\n");
        return (2);
    }

    // Get the regex result (0 if match, otherwise != 0)
    status = regexec(&re, string, (size_t) 0, NULL, 0);

    // Must free the compiled regular expression
    regfree(&re);

    if (status == 0)
    {
        return (0);      // successful validation
    }
    else
    {
        return (1);      // unsuccessful validation
    }
}
